
(*Mathematica:: Version 8.0      *)
(*:Context:: "markerfit`"        *)
(*:Title::        TrialFit *)
(*:References::                  *)
(*Date   2019 Stuart G. Baker *)
(*Input
GenStoredName	markerfitibasic.m
PlotFromModelFit     markerfitplot.m
TrialFitModel		markerfitmodel.m
GenDataSplit		markerfitsplit.m
TrialFitParameters	markerfitinputcheck.m*)



 (*Key Function 
 MarkerFit[dataset] where
     dataset={x0,x1,y0,y1,xname,datasetname} where
      x0 is g x n matrix of baseline variables for randomization group 0
      x1 is g x n matrix of baseline variables for randomization group 1
      y0 is length n list of binary outcomes (0 or 1) for randomization group 0
      y1 is length n list of binary outcomes (0 or 1) for randomization group 1
      xname= a length g list of names of baseline variables 
      datasetname=name of dataset*)
      
 
BeginPackage["markerfit`","markerfitinputcheck`",
"markerfitmodel`",
"markerfitRD`", "markerfitRDtrain`","markerfitRDtest`",
"markerfitresp`", "markerfitresptrain`","markerfitresptest`",
"markerfitbasic`","markerfitsplit`",
"markerfitlogit`", "markerfitplot`","markerfitboot`", "markerfitbootci`",
"markerfitsim`"]

MarkerFit::usage="TrialFit[data,options]"


Clear[MarkerFit]

Options[MarkerFit]=
 {FractionSplit->.5,
    BenefitFunction->"RespondersOnly5",   (*number is number of variables*)
  NewFitQ->True,
  ShowProgQ->False,
  ShowTabQ->True,
  MinimumSubgroupSize->20,
  NumCut->4,
  PlotFormat->"quantiles",
  NumFilter->20,
  MaxBoot->20}


BenefitFunction::usage="RespondersOnly,RiskDifference,or All"
NewFitQ::usage="true or false for using stored parameter estimates"
FractionSplit::usage="fraction split into test sample (between .2 and .8)"
MinimumSubgroupSize="miniumum sample size for subgroup"
NumCut::usage="number of cutpoints for subgroups in test sample (1 to 10) "
ShowProgQ::usage="show information about progress of fitting"
ShowTabQ::usage="show data and results tables"
NumFilter::usage="number in initial univariate filter"
MaxBoot::usage="number of bootstrap iterations for confidence intervals"
PlotFormat::usage=" quantiles or score for horzontal axis"

Begin["Private`"]	  


 MarkerFit[dataset_,options___Rule]:=
    Module[{optionvec,check,parameterset,method,split,datacore,bf,modelset,indexm,indexc,plotspec,
      plotinfoset,ymin,ymax},
         optionvec= {FractionSplit,NumCut,NumFilter,NewFitQ,
                             ShowProgQ,ShowTabQ, MaxBoot, 
                              MinimumSubgroupSize,PlotFormat,BenefitFunction}/. {options}/.Options[MarkerFit]; 
      (*MODEL SET*)
         bf=optionvec[[-1]];
         If[bf==="RespondersOnly1", modelset={{"Resp","F"}}];
         If[bf==="RiskDifference1",    modelset={{"RD","F"}}];
         If[bf==="RespondersOnly2", modelset={{"Resp","M"}}];
         If[bf==="RiskDifference2",    modelset={{"RD","M"}}];
         If[bf==="RespondersOnly5", modelset={{"Resp","X"}}];
         If[bf==="RiskDifference5",    modelset={{"RD","X"}}];
         If[bf==="All",                          modelset={{"RD","F"},{"RD","M"},{"RD","X"}, {"Resp","F"},{"Resp","M"},{"Resp","X"}}];
       
   
             
           Print["BenefitFunction and number of predictors: ",bf];
        lenmod=Length[modelset];
     (*check input and create sets of parameters*)
        {check,parameterset,plotspec}=TrialFitParameters[dataset,optionvec];
   (*computation*)  
    If[check,
          split=optionvec[[1]];
          datasetname=dataset[[-1]]; 
            seed=19;
           {datacore,datainfo}=GenDataSplit[dataset,split,seed];  
         plot= MarkerFitKey[dataset,datainfo,datacore,modelset,bf,parameterset,datasetname,plotspec];
       ExportPlot[datasetname,bf,plot]];  
     Return[Null]]
    


(*-------------------------------------------------------------------------*)
               
MarkerFitKey[dataset_,datainfo_,datacore_,modelset_,bf_,parameterset_,datasetname_,plotspec_]:= 
 Module[{input,method,newfitQ,fracmax,maxboot,tuningparameters,check,numcvsplits,numsplit,plotformat,
             yminvec,ymaxvec,plotvec0,ymin,ymax,plotvec,ncol,mat,
             plotmat,plot,plotvec1},
             {input,method,tuningparameters,check,numcvsplits,plotformat}=parameterset;
           numcut=input[[8]];
           fracmax=input[[9]];
           newfitQ=input[[10]];
          split=input[[12]];
          numsplit=Round[split 100];
          ReportRun[input,tuningparameters,datainfo];
          EvaluateAll[plotspec];
       (*FIT MODELS*)
            storedname=GenStoredNameX[datasetname,1,bf,numcut,numsplit];
	   If[newfitQ==False,
	                          mat=Quiet@Drop[ReadList[storedname],1][[1]];
	                           Print["   importing stored parameter estimates from ",storedname]];
	             If[newfitQ==True,
	                       Print["Fitting benefit functions"];
	                         mat=TrialFitModel[datacore,#,parameterset]& /@ modelset;   
	                        Print["    exporting training sample fits ",storedname];
                     Export[storedname,mat]];  
        (*PLOT*)
           If[bf=="All",ncol=3];
           If[bf=="RespondersOnly",ncol=2];       
           If[bf=="RiskDifference",ncol=2];       
           plot=PlotFromModelFit[mat,fracmax,plotspec,ncol,plotformat,"all"];       
 Return[plot]]  
 



 
 (*-------------------------------------------------Suppport-----------------------------------------------*)
     
  ExportPlot[datasetname_,bf_,plot_]:=
  Module[{plotname1,plotname2},
        Print[plot];
       plotname1=StringJoin["markerfit",datasetname,"FIG",bf,".eps"];
        Print["exporting ",plotname1];
  	Export[plotname1,plot];
  	plotname2=StringJoin["markerfit",datasetname,"FIG",bf,".jpeg"];
	 Print["exporting ",plotname2];
  	Export[plotname2,plot];
     Return[Null]]; 


ReportRun[input_,tuningparameters_,datainfo_]:=
Module[{fracmax,numfilter,maxboot,numcut,aucdif1,aucdif2,numchoose1,numchoose2,
             datasetname,nvar,nALL0,nALL1,split},
         fracmax=input[[9]];
        numfilter=input[[13]];
         maxboot=input[[2]];
         numcut=input[[8]];
        {aucdif1,aucdif2,numchoose1,numchoose2}= tuningparameters;
        {datasetname,nvar,nALL0,nALL1,split}=datainfo;
	 Print["DATA SET:  ",datasetname," sample  sizes:  ", {nALL0,nALL1}, " split= ",split, 
          " number of variables = ",nvar];
        Print["FITTING INFORMATION:   filter number=",numfilter, "  bootstrap number =",maxboot];
        Print["QUANTILES:  largest = ",fracmax, " number cutpoints= ",numcut];
    Return[Null]]



GenStoredNameX[datasetname_,method_,bfname_,numcut_,numsplit_]:=
Module[{numcuts,lenmods,methods,numsplits,storedname},
  If[bf=="All",                               bfname="A"];
	     If[bf=="RespondersOnly5",bfname="B5"];       
           If[bf=="RiskDifference5",    bfname="C5"];   
      numcuts=ToString[numcut];
      numsplits=ToString[numsplit];
      lenmods=ToString[lenmod];
         methods=ToString[method];
      storedname=StringJoin["trialSTORE", datasetname,methods,bfname,numcuts,numsplits,".m"];
    (* Print["STORED NAME ",storedname];*)
 Return[storedname]]  

End[] 
EndPackage[]

Print["MarkerFit[data]"];