(*Mathematica:: Version 8.0      *)
(*:Context:: "markerfit`"        *)
(*:Title::  plot  *)
(*:Summary::                     *)
(*:References::                  *)
(*Date   2014 Stuart G. Baker *)
 (*INPUT
 PlotInfoModelX   markerfitbasic*)
 
BeginPackage["markerfitplot`","markerfitbasic`","markerfitboot`", "markerfitbootci`"]  

PlotFromModelFit::usage="PlotFromModelFit"

Clear[PlotFromModelFit]

Begin["Private`"]	  



PlotFromModelFit[mat_,fracmax_,plotspec_,ncol_,plotformatx_,plotname_]:=
Module[{resset,xvecset,parset,pair,xvec,tri1,numcomp,plotinfovec,plotx,modvec,typevec,reportlist},
(*input*)
    {modelset,resset,xvecset,parset}=Transpose[mat];
(*print paramters*)
    pair=Transpose[{parset,modelset}];
    Print["PARAMETER ESTIMATES"];
    {modvec,typevec}=Transpose[modelset];
    reportlist=MemberQ[typevec,"Z"];
    lenz=Length[modelset];
    If[reportlist==False,
       matz= ReportParameter[parset,modelset,#]& /@ Range[lenz]; 
       matz1=Partition[Flatten[matz],9];
       colname={"func", "num", "grp", "0", "1","2", "3", "4", "5"};
       rowname=None;
       Print@TableForm[matz1,TableHeadings->{rowname,colname}]];
    If[reportlist==True, 
      PrintParameter[#]& /@pair];    
(*plot set*)
    xvec=xvecset[[1]];
  tri1=Transpose[{resset,modelset}];
  numcomp=Length[modelset];
    plotinfovec=PlotInfoModelX[#,fracmax,xvec,plotspec,numcomp]& /@ tri1;
 len=Length[modelset];
 If[len==1,    plotx=PlotOne[plotinfovec,ncol,plotformatx,Large]];
 If[len>1,      plotx=PlotGrid[plotinfovec,ncol,plotformatx,Medium]];
Return[plotx]]


(*------------------------*)

PlotInfoModelX[{resvec_,model_},fracmax_,xvec_,plotspec_,numcomp0_]:=   
        Module[{a,b,plotname,out,color},
          {a,b}=model;
         If[numcomp0==1,    color=Black,      color=GenColor[model]];
         plotname=Style[StringJoin[a,":",b],color];
         plotname=StringJoin[a,":",b];
      out= ReportTestSample[resvec,plotspec,{fracmax,xvec,plotname,color}];
        Return[out]] 



(*---------------------------------------------------*)

 PlotOne[plotinfovec_,ncol_,plotformatx_,size_]:=
 Module[{yminvec,ymaxvec,i,len,rem,nullvec,plotvecadd,plotvec,plotmat,plot},
    {yminvec,ymaxvec,plotvec0}=Transpose[plotinfovec];
      ymin=Min[yminvec];
     ymax=Max[ymaxvec];
     plot1=plotvec0[[1]];
        plotvec=PlotCut[plot1,{ymin,ymax},"score",0,ncol,size,"one"];
       plot=Show[plotvec,ImageSize->Large];
    Return[plot]]

(*---------------------------------------------------*)


 PlotGrid[plotinfovec_,ncol_,plotformatx_,size_]:=
 Module[{yminvec,ymaxvec,i,len,rem,nullvec,plotvecadd,plotvec,plotmat,plot},
    {yminvec,ymaxvec,plotvec0}=Transpose[plotinfovec];
      ymin=Min[yminvec];
     ymax=Max[ymaxvec];
         plotvec=PlotCut[#,{ymin,ymax},plotformatx,1,ncol,size,"many"]& /@ plotvec0;
        plot=PlotGridFormat[plotvec,ncol];
    Return[plot]]
 
 PlotGridFormat[plotvec_,ncol_]:=
 Module[{len,rem,plotvecadd,nullvec,plotmat,plot},
       len=Length[plotvec];
      rem=Mod[len,ncol];
     If[rem==0,
         plotvecadd=plotvec,
         nullvec=Table[Null,{i,1,ncol-rem}];
        plotvecadd=Join[plotvec,nullvec]];
     (*partition*)
     plotmat=Partition[plotvecadd,ncol];
      plot=Show[GraphicsGrid[plotmat],ImageSize->Large];
 Return[plot]]
 
(*---------------------------------------------------*)


PlotCut[{inputvec_,dif0_,low0_,upp0_,threshold_,zboot_,plotname_,qvec_,color0_},{ymin_,ymax_},
plotformat_,format_,ncol_,size_,set_]:=
Module[{difvec,fracvec,sevec,m0,m1,n0,n1,e,dmin,dmax,
            lineplot,lowvec,uppvec,lowvec0,uppvec0,xticks,rangex,ylabelx,
            pointTEST,lineTEST,pointTESTlow,pointTESTupp,
            plotcore,plot},
 (*BASIC  QUANTITITES*)           
 color=Black;
 th=.01; 
 
 {xvec0,fracvec,difvec,sevec,m0,m1,n0,n1}=inputvec;
        If[plotformat=="quantiles",xvec=qvec];
       If[plotformat=="score", xvec=xvec0];
        (*TEST SAMPLE POINTS *)
          lowvec0=difvec-zboot sevec;
          uppvec0=difvec+zboot sevec;
       (*adjust for boundary values*)
            lowvec=LowVec[lowvec0];
           uppvec=UppVec[uppvec0];
      (*connecting lines*)
         lineplot=LineSet[xvec,difvec,"est",color,th];
         lineplotlow=LineSet[xvec,lowvec,"ci",color,th];
         lineplotupp=LineSet[xvec,uppvec,"ci",color,th];
      (*points*)    
         psize=Medium;
         pointTEST=Graphics[{color,PointSize[psize],Point[#]}]& /@ Transpose[{xvec,difvec}];
         pointTESTlow=Graphics[{color,PointSize[psize],Point[#]}]& /@ Transpose[{xvec,lowvec}];
         pointTESTupp=Graphics[{color,PointSize[psize],Point[#]}]& /@ Transpose[{xvec,uppvec}];
      (*line for overall treatment effect*)    
       dmin=Min[xvec];
       dmax=Max[xvec];
       (*overall effect*)
        hline=Graphics[{Dashing[{.01,.01}],Line[{{dmin,dif0},{dmax,dif0}}]}];
        zeroline=Graphics@Line[{{dmin,0},{dmax,0}}];
     (*PLOT*)    
        (*if sample size label is desired on plot--not used*)
          e=Max[sevec/5];
          textset=TextSize[#,e]& /@ Transpose[{xvec,difvec,m0,m1,n0,n1}];
       (*plot*)
         maxqvec=Max[qvec];
         maxqvec0=maxqvec 1.01;
          maxxvec=Max[xvec];
          minxvec=Min[xvec];
         maxxvec0=maxxvec+ .01 (maxxvec-minxvec);
         minxvec0=minxvec- .01 (maxxvec-minxvec);
          If[plotformat=="quantiles",
            xlabel="Quantile (benefit score)";   
           rangex={0,maxqvec0}; 
            xticks=GenTicksForQuant[xvec0,qvec,ncol]];
         If[plotformat=="score",   
             xlabel="Benefit score";       
                rangex={0,maxqvec0}; 	
	                      xticks=Automatic];
                   plotcore={pointTEST, pointTESTlow,pointTESTupp,lineplot,lineplotlow,lineplotupp,zeroline};
              plotcore={lineplot,lineplotlow,lineplotupp,zeroline};
           plotnamex=Style[plotname,color];
            xlabelx=Style[xlabel,Black,size];
            If[set=="one",              ylabelx=Style["Difference in subgroup",Black,size]];
            If[set=="many",           ylabelx=Style["Difference",Black,size]];
            
             If[plotname=="RD:F",    topnameNEW="Risk Difference\n with one marker"];
             If[plotname=="Resp:F",  topnameNEW="Responder-Only\n with one marker"];          
             If[plotname=="RD:M",    topnameNEW="Risk Difference\n with two markers"];
	       If[plotname=="Resp:M",  topnameNEW="Responder-Only\n with two markers"];                   
             If[plotname=="RD:X",    topnameNEW="Risk Difference\n with five markers"];
	      If[plotname=="Resp:X",  topnameNEW="Responder-Only\n with five markers"];               
               If[plotname=="RD:Z",    topnameNEW="Risk Difference\n with ten markers"];
	       If[plotname=="Resp:Z",  topnameNEW="Responder-Only\n with ten markers"];               
         
        If[set=="many", topnameNEWx=Style[topnameNEW,Medium,Black]];
        If[set=="one", topnameNEWx="Multivariate STEPP"];
          plot=Show[plotcore,
               Frame->True,
               FrameTicks->{xticks,Automatic,None,None},
               PlotRange->{Automatic,{ymin,ymax}},
               LabelStyle->{Black,size},	
               FrameLabel->{xlabelx,ylabelx,topnameNEWx,None},AspectRatio->1];                     
   Return[plot]]      
  

    
       (*------------------------------------------------*) 
      
         

  GenTicksForQuant[xvec_,qvec_,ncol_]:=
  Module[{maxxvec,minvec,xmax,xmin,xmid,xmaxr,xminr,xmidr,xmaxs,xmids,xmins,
      qmax,qmin,qmid, qmaxr,qmidr,qminr,qmaxs,qmids,qmins,qticks,zticks,xticks},
            xmax=Max[xvec];
            xmin=Min[xvec];
            xmid=(xmax+xmin)/2;
            
            xmaxr=Round[xmax 100]/100//N;
            xminr=Round[xmin 100]/100//N;
            xmidr=Round[xmid 100]/100//N;
            xmaxs=ToString[xmaxr];
            xmids=ToString[xmidr];
            xmins=ToString[xminr];
           qmax=Max[qvec];
           qmid= qmax/2;
            qmin=0;
            qticks={qmin,qmid,qmax};
                 qmaxr=Round[qmax 100]/100//N;
	                qminr=Round[qmin 100]/100//N;
	                qmidr=Round[qmid 100]/100//N;
	                qmaxs=ToString[qmaxr];
	                qmids=ToString[qmidr];
	                qmins=ToString[qminr];
      If[ncol<=3,	                
       qticks={qminr,qmidr,qmaxr};
       zticks={StringJoin[qmins,"\n(",xmins,")"],StringJoin[qmids,"\n(",xmids,")"],
                        StringJoin[qmaxs,"\n(",xmaxs,")"]}];
      If[ncol>3,	                
       qticks={qminr,qmaxr};
       zticks={StringJoin[qmins,"\n(",xmins,")"], StringJoin[qmaxs,"\n(",xmaxs,")"]}];
                        
                xticks=Transpose[{qticks,zticks}];
  Return[xticks]]
  
  
   GenTicksForQuantOnly[qvec_,ncol_]:=
    Module[{maxxvec,minvec,xmax,xmin,xmid,xmaxr,xminr,xmidr,xmaxs,xmids,xmins,
        qmax,qmin,qmid, qmaxr,qmidr,qminr,qmaxs,qmids,qmins,qticks,zticks,xticks},
             qmax=Max[qvec];
             qmid= qmax/2;
              qmin=0;
              qticks={qmin,qmid,qmax};
                   qmaxr=Round[qmax 100]/100//N;
  	                qminr=Round[qmin 100]/100//N;
  	                qmidr=Round[qmid 100]/100//N;
  	                qmaxs=ToString[qmaxr];
  	                qmids=ToString[qmidr];
  	                qmins=ToString[qminr];
        If[ncol<=3,	                
         qticks={qminr,qmidr,qmaxr}];
          If[ncol>3,	                
         qticks={qminr,qmaxr}];
    Return[qticks]]
  
  
     TextSize[{d_,dif_,m0_,m1_,n0_,n1_},e_]:=
     Module[{m0s,m1s,name,res},
         m0s=ToString[m0];
         m1s=ToString[m1];
        name=StringJoin["(",m0s,",",m1s,")"];
        ns=ToString@Min[n0,n1];
        name=StringJoin["(",ns,")"];
        res=Graphics@Text[Style[name,FontSize->12],{d-e,dif+e},{0,-1}];
     Return[res]]
  
     UppVec[vec_]:= Min[1,#]&  /@ vec
     LowVec[vec_]:= Max[-1,#]&  /@ vec

(*create line sements*)

LineSet[xvec_,yvec_,type_,color_,th_]:=
Module[{pairvec,pairvec1,lineset},
   pairvec=Transpose[{xvec,yvec}];
   pairvec1=Partition[pairvec,2,1];
   If[type=="est",  lineset=LinePairEst[#,color,th]& /@ pairvec1];
   If[type=="ci",  lineset=LinePairCI[#,color,th]& /@ pairvec1];
 Return[lineset]]

LinePairEst[{{a_,b_},{c_,d_}},color_,th_]:= Graphics@{color,Thickness[th],Line[{{a,b},{c,d}}]}
LinePairCI[{{a_,b_},{c_,d_}},color_,th_]:= Graphics@{color,Thickness[th],Dashing[{.03,.03}],Line[{{a,b},{c,d}}]}


  
(*------------------------------*)
 

ReportParameter[parset_,modelset_,j_]:=
Module[{len,mod,var1,num1,mat1,var2,num2,mat2, var,num,mat,i,blank},
   mod=modelset[[j,1]];
   type=modelset[[j,2]];
   If[type=="F", len=1];
   If[type=="M", len=2];
   If[type=="X", len=5];
      If[type=="Z", len=10];   
 If[mod=="RD",
   var1=parset[[j,1,1,1]];
  num1=parset[[j,1,1,2]];
  mat1=ParCore[var1,num1,"RD",len,1];
  var2=parset[[j, 2,1,1]];
  num2=parset[[j,2,1,2]];
  mat2=ParCore[var2,num2,"RD",len,2];
  mat=Join[mat1,mat2]];
  If[mod=="Resp",
    var=parset[[j,1,1,1]];
    num=parset[[j,1,1,2]];
    mat=ParCore[var,num,"Resp",len,0]];
  Return[mat]];
  
  
  
   
   ParCore[var_,num0_,mod_,k_,row_]:=
   Module[{len,var1,num1,blank,i},
   num=PaddedForm[#,3]& /@num0;
    len=Length[var];
       If[len == 6,
     var1=Join[{mod,k,row},var];
     num1=Join[{mod,k,row},num]];
     If[len<6,
      zero=Table[0,{i,1,6-len}];
      blank=Table[" ",{i,1,6-len}];
      num1=Join[{mod,k,row},num,blank];
      var1=Join[{mod,k,row},var,blank]];
   mat={var1,num1};
   Return[mat]]
   


  
 
  
   
End[] 
EndPackage[]
